#!/bin/sh
#@Title         PowerVR SGX DDK for Embedded Linux 
#@Copyright     Copyright (c) Imagination Technologies Ltd. All Rights Reserved
#@License       Strictly Confidential.

# Auto-generated for omap4430_linux from build: sgxddk_1.9.RC2\@2139099
#

# OMAP version number
#
OMAPVERSION=1.9.6.0

# Where we record what we did so we can undo it.
#
DDK_INSTALL_LOG=/etc/powervr_ddk_install.log

# basic installation function
# $1=blurb
#
bail()
{
    echo "$1" >&2
    echo "" >&2
    echo "Installation failed" >&2
    exit 1
}

# basic installation function
# $1=fromfile, $2=destfilename, $3=blurb, $4=chmod-flags, $5=chown-flags
#
install_file()
{
	if [ ! -e $1 ]; then
	 	[ -n "$VERBOSE" ] && echo "skipping file $1 -> $2"
		 return
	fi
	
	DESTFILE=${DISCIMAGE}$2
	DESTDIR=`dirname $DESTFILE`

	$DOIT mkdir -p ${DESTDIR} || bail "Couldn't mkdir -p ${DESTDIR}"
	[ -n "$VERBOSE" ] && echo "Created directory `dirname $2`"

	# Delete the original so that permissions don't persist.
	$DOIT rm -f $DESTFILE
	$DOIT cp -f $1 $DESTFILE || bail "Couldn't copy $1 to $DESTFILE"
	$DOIT chmod $4 ${DISCIMAGE}$2
	$DOIT chown $5 ${DISCIMAGE}$2

	echo "$3 `basename $1` -> $2"
	$DOIT echo "file $2" >>${DISCIMAGE}${DDK_INSTALL_LOG}
}

# Install a symbolic link
# $1=fromfile, $2=destfilename
#
install_link()
{
	DESTFILE=${DISCIMAGE}$2
	DESTDIR=`dirname $DESTFILE`

	if [ ! -e ${DESTDIR}/$1 ]; then
		 [ -n "$VERBOSE" ] && echo $DOIT "skipping link ${DESTDIR}/$1"
		 return
	fi

	$DOIT mkdir -p ${DESTDIR} || bail "Couldn't mkdir -p ${DESTDIR}"
	[ -n "$VERBOSE" ] && echo "Created directory `dirname $2`"

	# Delete the original so that permissions don't persist.
	#
	$DOIT rm -f $DESTFILE

	$DOIT ln -s $1 $DESTFILE || bail "Couldn't link $1 to $DESTFILE"
	$DOIT echo "link $2" >>${DISCIMAGE}${DDK_INSTALL_LOG}
	[ -n "$VERBOSE" ] && echo " linked `basename $1` -> $2"
}

# Tree-based installation function
# $1 = fromdir $2=destdir $3=blurb
#
install_tree()
{
	if [ ! -z $INSTALL_TARGET ]; then
		# Use rsync and SSH to do the copy as it is way faster.
		echo "rsyncing $3 to root@$INSTALL_TARGET:$2"
		$DOIT rsync -crlpt -e ssh $1/* root@$INSTALL_TARGET:$2 || bail "Couldn't rsync $1 to root@$INSTALL_TARGET:$2"
	else 
		$DOIT mkdir -p ${DISCIMAGE}$2 || bail "Couldn't mkdir -p ${DISCIMAGE}$2"
		if [ -z "$DOIT" ]; then
			tar -C $1 -cf - . | tar -C ${DISCIMAGE}$2 -x${VERBOSE}f -
		else
			$DOIT "tar -C $1 -cf - . | tar -C ${DISCIMAGE}$2 -x${VERBOSE}f -"
		fi
	fi

	if [ $? = 0 ]; then
		echo "Installed $3 in ${DISCIMAGE}$2"
		$DOIT echo "tree $2" >>${DISCIMAGE}${DDK_INSTALL_LOG}
	else
		echo "Failed copying $3 from $1 to ${DISCIMAGE}$2"
	fi
}

# Uninstall something.
#
uninstall()
{
	if [ ! -f ${DISCIMAGE}${DDK_INSTALL_LOG} ]; then
		echo "Nothing to un-install."
		return;
	fi

	BAD=0
	VERSION=""
	while read type data; do
		case $type in
		version)	# do nothing
			echo "Uninstalling existing version $data"
			VERSION="$data"
			;;
		link|file) 
			if [ -z "$VERSION" ]; then
				BAD=1;
				echo "No version record at head of ${DISCIMAGE}${DDK_INSTALL_LOG}"
			elif ! $DOIT rm -f ${DISCIMAGE}${data}; then
				BAD=1;
			else
				[ -n "$VERBOSE" ] && echo "Deleted $type $data"
			fi
			;;
		tree)		# so far, do nothing
			;;
		esac
	done < ${DISCIMAGE}${DDK_INSTALL_LOG};

	if [ $BAD = 0 ]; then
		echo "Uninstallation completed."
		$DOIT rm -f ${DISCIMAGE}${DDK_INSTALL_LOG}
	else
		echo "Uninstallation failed!!!"
	fi
}

# Help on how to invoke
#
usage()
{
	echo "usage: $0 [options...]"
	echo ""
	echo "Options: -v            verbose mode"
	echo "         -n            dry-run mode"
	echo "         -u            uninstall-only mode"
	echo "         --no-pvr      don't install PowerVR driver components"
	echo "         --no-x        don't install X window system"
	echo "         --no-display  don't install integrated PowerVR display module"
	echo "         --no-bcdevice don't install buffer class device module"
	echo "         --root path   use path as the root of the install file system"
	exit 1
}

install_pvr()
{
	$DOIT echo "version 1.9.6.0" >${DISCIMAGE}${DDK_INSTALL_LOG}
	# Install the standard scripts
	#
	install_file rc.pvr /etc/init.d/rc.pvr "boot script" 0755 0:0

	# Install the standard pkgconfig files
	#
	install_file egl.pc /usr/lib/pkgconfig/egl.pc "pkgconfig" 0644 0:0
	install_file glesv1_cm.pc /usr/lib/pkgconfig/glesv1_cm.pc "pkgconfig" 0644 0:0
	install_file glesv2.pc /usr/lib/pkgconfig/glesv2.pc "pkgconfig" 0644 0:0
	install_file wayland-egl.pc /usr/lib/pkgconfig/wayland-egl.pc "pkgconfig" 0644 0:0
	# Install the standard libraries
	#
	install_file libGLESv1_CM.so /usr/lib/libGLESv1_CM.so.1.9.6.0 "shared library" 0644 0:0
	install_link libGLESv1_CM.so.1.9.6.0 /usr/lib/libGLESv1_CM.so.1
	install_link libGLESv1_CM.so.1 /usr/lib/libGLESv1_CM.so


	install_file libusc.so /usr/lib/libusc.so.1.9.6.0 "shared library" 0644 0:0
	install_link libusc.so.1.9.6.0 /usr/lib/libusc.so

	install_file libGLESv2.so /usr/lib/libGLESv2.so.1.9.6.0 "shared library" 0644 0:0
	install_link libGLESv2.so.1.9.6.0 /usr/lib/libGLESv2.so.2
	install_link libGLESv2.so.2 /usr/lib/libGLESv2.so

	install_file libglslcompiler.so /usr/lib/libglslcompiler.so.1.9.6.0 "shared library" 0644 0:0
	install_link libglslcompiler.so.1.9.6.0 /usr/lib/libglslcompiler.so



	install_file libIMGegl.so /usr/lib/libIMGegl.so.1.9.6.0 "shared library" 0644 0:0
	install_link libIMGegl.so.1.9.6.0 /usr/lib/libIMGegl.so
	install_file libEGL.so /usr/lib/libEGL.so.1.9.6.0 "shared library" 0644 0:0
	install_link libEGL.so.1.9.6.0 /usr/lib/libEGL.so.1
	install_link libEGL.so.1 /usr/lib/libEGL.so
	install_file libpvr2d.so /usr/lib/libpvr2d.so.1.9.6.0 "shared library" 0644 0:0
	install_link libpvr2d.so.1.9.6.0 /usr/lib/libpvr2d.so


	install_file libpvrws_OMAPDRI2.so /usr/lib/libpvrws_OMAPDRI2.so.1.9.6.0 "shared library" 0644 0:0
	install_link libpvrws_OMAPDRI2.so.1.9.6.0 /usr/lib/libpvrws_OMAPDRI2.so
	install_file libpvrws_KMS.so /usr/lib/libpvrws_KMS.so.1.9.6.0 "shared library" 0644 0:0
	install_link libpvrws_KMS.so.1.9.6.0 /usr/lib/libpvrws_KMS.so
	install_file libpvrws_WAYLAND.so /usr/lib/libpvrws_WAYLAND.so.1.9.6.0 "shared library" 0644 0:0
	install_link libpvrws_WAYLAND.so.1.9.6.0 /usr/lib/libpvrws_WAYLAND.so
	install_file libpvr_wlegl.so /usr/lib/libpvr_wlegl.so.1.9.6.0 "shared library" 0644 0:0
	install_link libpvr_wlegl.so.1.9.6.0 /usr/lib/libpvr_wlegl.so
	install_file gbm_pvr.so /usr/lib/gbm_pvr.so.1.9.6.0 "shared library" 0644 0:0
	install_link gbm_pvr.so.1.9.6.0 /usr/lib/gbm_pvr.so

	install_file libsrv_um.so /usr/lib/libsrv_um.so.1.9.6.0 "shared library" 0644 0:0
	install_link libsrv_um.so.1.9.6.0 /usr/lib/libsrv_um.so
	install_file libsrv_init.so /usr/lib/libsrv_init.so.1.9.6.0 "shared library" 0644 0:0
	install_link libsrv_init.so.1.9.6.0 /usr/lib/libsrv_init.so
	install_file libPVRScopeServices.so /usr/lib/libPVRScopeServices.so.1.9.6.0 "shared library" 0644 0:0
	install_link libPVRScopeServices.so.1.9.6.0 /usr/lib/libPVRScopeServices.so


	install_file libegl4ogl.so /usr/lib/libegl4ogl.so.1.9.6.0 "shared library" 0644 0:0
	install_link libegl4ogl.so.1.9.6.0 /usr/lib/libegl4ogl.so



	# Install the standard executables
	#

	install_file pvrsrvinit /usr/bin/pvrsrvinit "binary" 0755 0:0

	install_file sgx_init_test /usr/bin/sgx_init_test "binary" 0755 0:0
	install_file pvrsrvctl /usr/bin/pvrsrvctl "binary" 0755 0:0



	# Install the standard headers
	#
	install_file eglext.h /usr/include/EGL/eglext.h "header" 0644 0:0
	install_file egl.h /usr/include/EGL/egl.h "header" 0644 0:0
	install_file eglplatform.h /usr/include/EGL/eglplatform.h "header" 0644 0:0
	install_file khrplatform.h /usr/include/KHR/khrplatform.h "header" 0644 0:0
	install_file glext.h /usr/include/GLES/glext.h "header" 0644 0:0
	install_file gl.h /usr/include/GLES/gl.h "header" 0644 0:0
	install_file glplatform.h /usr/include/GLES/glplatform.h "header" 0644 0:0
	install_file gl2ext.h /usr/include/GLES2/gl2ext.h "header" 0644 0:0
	install_file gl2.h /usr/include/GLES2/gl2.h "header" 0644 0:0
	install_file gl2platform.h /usr/include/GLES2/gl2platform.h "header" 0644 0:0
	# Install the standard unittests
	#








	install_file xeglinfo /usr/bin/xeglinfo "binary" 0755 0:0

	install_file xgles1test1 /usr/bin/xgles1test1 "binary" 0755 0:0
	install_file xmultiegltest /usr/bin/xmultiegltest "binary" 0755 0:0

	install_file xgles2test1 /usr/bin/xgles2test1 "binary" 0755 0:0
	install_file glsltest1_vertshader.txt /usr/share/sgx-lib/glsltest1_vertshader.txt "shader" 0644 0:0
	install_file glsltest1_fragshaderA.txt /usr/share/sgx-lib/glsltest1_fragshaderA.txt "shader" 0644 0:0
	install_file glsltest1_fragshaderB.txt /usr/share/sgx-lib/glsltest1_fragshaderB.txt "shader" 0644 0:0


}

install_X()
{


	[ -d usr ] &&
		install_tree usr/local/XSGX /usr/local/XSGX "X.Org X server and libraries"
	[ -f pvr_drv.so ] &&
		install_file pvr_drv.so /usr/lib/xorg/modules/drivers/pvr_drv.so "X.Org PVR DDX module" 0755 0:0
 	[ -f xorg.conf ] &&
		install_file xorg.conf /usr/etc/xorg.conf "X.Org configuration file" 0644 0:0
  }

# Work out if there are any special instructions.
#
while [ "$1" ]; do
	case "$1" in
	-v|--verbose)
		VERBOSE=v;
		;;
	-r|--root)
		DISCIMAGE=$2;
		shift;
		;;
	-t|--install-target)
		INSTALL_TARGET=$2;
		shift;
		;;
	-u|--uninstall)
		UNINSTALL=y
		;;
	-n)	DOIT=echo
		;;
	--no-pvr)
		NO_PVR=y
		;;
	--no-x)
		NO_X=y
		;;
	--no-display)
		NO_DISPLAYMOD=y
		;;
	--no-bcdevice)
		NO_BCDEVICE=y
		;;
	-h | --help | *)	
		usage
		;;
	esac
	shift
done

# Find out where we are?  On the target?  On the host?
#
case `uname -m` in
arm*)	host=0;
		from=target;
		# Allow 'DISCIMAGE' override for native build.
		# Revert to '/' only if unset.
		[ -z "$DISCIMAGE" ] && DISCIMAGE=/;
		;;
sh*)	host=0;
		from=target;
		DISCIMAGE=/;
		;;
i?86*)	host=1;
		from=host;
		if [ -z "$DISCIMAGE" ]; then	
			echo "DISCIMAGE must be set for installation to be possible." >&2
			exit 1
		fi
		;;
x86_64*)	host=1;
		from=host;
		if [ -z "$DISCIMAGE" ]; then	
			echo "DISCIMAGE must be set for installation to be possible." >&2
			exit 1
		fi
		;;
*)		echo "Don't know host to perform on machine type `uname -m`" >&2;
		exit 1
		;;
esac

if [ ! -z "$INSTALL_TARGET" ]; then
	if ssh -q -o "BatchMode=yes" root@$INSTALL_TARGET "test 1"; then
		echo "Using rsync/ssh to install to $INSTALL_TARGET"
	else
		echo "Can't access $INSTALL_TARGET via ssh."
		# We have to use the `whoami` trick as this script is often run with 
		# sudo -E
		if [ ! -e ~`whoami`/.ssh/id_rsa.pub ] ; then
			echo " You need to generate a public key for root via ssh-keygen"
			echo " then append it to root@$INSTALL_TARGET:~/.ssh/authorized_keys."
		else
			echo "Have you installed root's public key into root@$INSTALL_TARGET:~/.ssh/authorized_keys?"
			echo "You can do so by executing the following as root:"
			echo "ssh root@$INSTALL_TARGET \"mkdir -p .ssh; cat >> .ssh/authorized_keys\" < ~/.ssh/id_rsa.pub"
		fi
		echo "Falling back to copy method."
		unset INSTALL_TARGET
	fi
fi

if [ ! -d "$DISCIMAGE" ]; then
	echo "$0: $DISCIMAGE does not exist." >&2
	exit 1
fi

echo
echo "Installing PowerVR Consumer/Embedded DDK 1.9.6.0 on $from"
echo
echo "File system installation root is $DISCIMAGE"
echo

# Uninstall whatever's there already.
#
uninstall
[ -n "$UNINSTALL" ] && exit

#  Now start installing things we want.
#
[ -z "$NO_PVR" ] && install_pvr
[ -z "$NO_X" ] && install_X

# All done...
#
echo 
echo "Installation complete!"
if [ "$host" = 0 ]; then
   echo "You may now reboot your target."
fi
echo
